from .. import TestBase


class UnitTestBase(TestBase):
    """
    Base class for ExParser unit tests.
    """

    def setUp(self):
        super(UnitTestBase, self).setUp()

    def tearDown(self):
        super(UnitTestBase, self).setUp()
